/**
* @typedef {import('./project.mjs').Project} Project
* @typedef {import('./milestone.mjs').Milestone} Milestone
*/
/**
 * @typedef {Object} MessageDestination
 * Endpoint to deliver log messages to.
 * @property {Function} info
 * @property {Function} debug
 * @property {Function} warn
 * @property {Function} error
 * @property {Function} trace
 */
/**
 * @property {MessageDestination} messageDestination
 */
export class BaseProvider extends BaseObject {
    /**
     * Prefix used to form environment variables.
     * 'GITHUB_' -> 'GITHUB_TOKEN'
     * @return {string} identifier for environment options
     */
    static get instanceIdentifier(): string;
    /**
     * Extract options suitable for the constructor.
     * Form the given set of environment variables.
     * Object with the detected key value pairs is delivered.
     * @param {Object} [env] as from process.env
     * @param {string} instanceIdentifier part of variable name.
     * @return {Object|undefined} undefined if no suitable environment variables have been found
     */
    static optionsFromEnvironment(env?: any, instanceIdentifier?: string): any | undefined;
    /**
     * Check if given options are sufficient to create a provider.
     * @param {Object} options
     * @return {boolean} true if options ar sufficient to construct a provider
     */
    static areOptionsSufficcient(options: any): boolean;
    static get attributes(): {
        /**
         * Name of the provider.
         */
        name: {
            env: string;
            type: string;
            isKey: boolean;
            writable: boolean;
            mandatory: boolean;
            private?: boolean;
            depends?: string;
            additionalAttributes: string[];
            description?: string;
            default?: any;
            set?: Function;
            get?: Function;
        };
        url: import("pacc").AttributeDefinition;
        description: import("pacc").AttributeDefinition;
        priority: import("pacc").AttributeDefinition;
        /**
         * To forward info/warn and error messages to
         */
        messageDestination: {
            type: string;
            default: Console;
            writable: boolean;
            private: boolean;
            isKey: boolean;
            mandatory: boolean;
            depends?: string;
            additionalAttributes: string[];
            description?: string;
            set?: Function;
            get?: Function;
            env?: string | string[];
        };
    };
    /**
     * Creates a new provider for a given set of options.
     * @param {Object} options additional options
     * @param {string} [options.instanceIdentifier] name of the provider instance
     * @param {Object} env taken from process.env
     * @return {BaseProvider|undefined} newly created provider or undefined if options are not sufficient to construct a provider
     */
    static initialize(options: {
        instanceIdentifier?: string;
    }, env: any): BaseProvider | undefined;
    get priority(): number;
    /**
     * @param {any} other
     * @return {boolean} true if other provider is the same as the receiver
     */
    equals(other: any): boolean;
    /**
     * All supported base urls.
     * For github something like:
     * - git@github.com
     * - git://github.com
     * - git+ssh://github.com
     * - https://github.com
     * - git+https://github.com
     * By default we provide provider name with ':'.
     * @return {string[]} common base urls of all repositories
     */
    get repositoryBases(): string[];
    /**
     * Does the provider support the base name.
     * @param {string} [base] to be checked
     * @return {boolean} true if base is supported or base is undefined
     */
    supportsBase(base?: string): boolean;
    /**
     * Bring a repository name into its normal form by removing any clutter.
     * Like .git suffix or #branch names.
     * @param {string} name
     * @param {boolean} forLookup
     * @return {string} normalized name
     */
    normalizeRepositoryName(name: string, forLookup: boolean): string;
    /**
     * Bring a group name into its normal form by removing any clutter.
     * Like .git suffix or #branch names.
     * @param {string} name
     * @param {boolean} forLookup
     * @return {string} normalized name
     */
    normalizeGroupName(name: string, forLookup: boolean): string;
    /**
     * Are repository names case sensitive.
     * Overwrite and return false if you want to have case insensitive repository lookup.
     * @return {boolean} true
     */
    get areRepositoryNamesCaseSensitive(): boolean;
    /**
     * Are repositroy group names case sensitive.
     * Overwrite and return false if you want to have case insensitive group lookup.
     * @return {boolean} true
     */
    get areGroupNamesCaseSensitive(): boolean;
    /**
     * Parses repository name and tries to split it into
     * base, group, repository and branch.
     * @param {string} [name]
     * @param {string} focus where lies the focus if only one path component is given
     * @return {Object} with separated attributes
     */
    parseName(name?: string, focus?: string): any;
    /**
     * Create a repository.
     * @param {string} name of group and repository
     * @param {Object} [options]
     * @returns {Promise<Repository>}
     */
    createRepository(name: string, options?: any): Promise<Repository>;
    /**
     * List provider objects of a given type.
     *
     * @param {string} type name of the method to deliver typed iterator projects,milestones,hooks,repositories,branches,tags
     * @param {string[]|string|undefined} patterns group / repository filter
     */
    list(type: string, patterns: string[] | string | undefined): AsyncGenerator<any, void, any>;
    /**
     * List projects.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterator<Project>} all matching projects of the provider
     */
    projects(patterns?: string[] | string): AsyncIterator<Project>;
    /**
     * List milestones.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterator<Milestone>} all matching milestones of the provider
     */
    milestones(patterns?: string[] | string): AsyncIterator<Milestone>;
    /**
     * List repositories.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterable<Repository>} all matching repos of the provider
     */
    repositories(patterns?: string[] | string): AsyncIterable<Repository>;
    /**
     * List branches.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterable<Branch>} all matching branches of the provider
     */
    branches(patterns?: string[] | string): AsyncIterable<Branch>;
    /**
     * List tags.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterable<Tag>} all matching tags of the provider
     */
    tags(patterns?: string[] | string): AsyncIterable<Tag>;
    /**
     * List hooks.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterable<Hook>} all matching hooks of the provider
     */
    hooks(patterns?: string[] | string): AsyncIterable<Hook>;
    /**
     * List pull requests.
     * @param {string[]|string} [patterns]
     * @return {AsyncIterable<PullRequest>} all matching pullRequests of the provider
     */
    pullRequests(patterns?: string[] | string): AsyncIterable<PullRequest>;
    /**
     * Deliver the provider name.
     * @return {string} class name by default
     */
    get name(): string;
    /**
     * We are our own provider.
     * @return {BaseProvider} this
     */
    get provider(): BaseProvider;
    /**
     * List all defined entries from attributes.
     * return {object}
     */
    toJSON(): {
        name: string;
    };
    initializeRepositories(): void;
    trace(...args: any[]): any;
    debug(...args: any[]): any;
    info(...args: any[]): any;
    warn(...args: any[]): any;
    error(...args: any[]): any;
    get repositoryGroupClass(): typeof RepositoryGroup;
    get hookClass(): typeof Hook;
    get repositoryClass(): typeof Repository;
    get branchClass(): typeof Branch;
    get tagClass(): typeof Tag;
    get entryClass(): typeof ContentEntry;
    get pullRequestClass(): typeof PullRequest;
}
export type Project = import('./project.mjs').Project;
export type Milestone = import('./milestone.mjs').Milestone;
/**
 * Endpoint to deliver log messages to.
 */
export type MessageDestination = {
    info: Function;
    debug: Function;
    warn: Function;
    error: Function;
    trace: Function;
};
import { BaseObject } from "./base-object.mjs";
import { Repository } from "./repository.mjs";
import { Branch } from "./branch.mjs";
import { Tag } from "./tag.mjs";
import { Hook } from "./hook.mjs";
import { PullRequest } from "./pull-request.mjs";
import { RepositoryGroup } from "./repository-group.mjs";
import { ContentEntry } from "content-entry";
